'use strict';

const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const _ = require('lodash');
const rename = require("gulp-rename");
const store = require('mem-fs').create();
const program = require('ast-query');
const util = require('../../misc/util.js');

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts);

        this.argument('pageName', { type: String, required: false, desc: '页面名称，一些文件名也会基于此而生成' });
    }

    initializing() {
        this.options.repoType = util.checkCwd(this);
    }

    //没有通过命令行传递页面名，或者传递的页面名与现有页面冲突，则显示页面输入框提示输入
    _showPageNamePrompt() {
        return !this.options.pageName
            || util.isPageExists(this.fs, this.options.repoType, this.options.pageName);
    }

    prompting() {
        this.log(yosay('欢迎使用 ' + chalk.red('umu') + ' 模板代码生成器'));

        const prompts = [{
            type: 'input',
            name: 'author',
            store: true,
            message: '作者（用于生成文件头部的文档）:',
            default: util.getUserName(this)
        }, {
            type: 'input',
            name: 'pageName',
            store: true,
            message: '页面名称，一些文件名也会基于此而生成',
            default: this.options.pageName || '',
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                return this._showPageNamePrompt();
            },
            validate: (input, answers) => {
                if (!_.trim(input)) {
                    return chalk.red('页面名不能为空！');
                } else if (util.isPageExists(this.fs, this.options.repoType, input)) {
                    return chalk.red(`${input} 已经存在`);
                } else {
                    return true;
                }
            }
        }];

        return this.prompt(prompts).then(props => {
            this.options = Object.assign({}, this.options, props);

            this.options.pageName = _.camelCase(this.options.pageName);
            //中划线形式的名称,用于样式类名
            this.options.pageNameClsForm = _.kebabCase(this.options.pageName);
            //大写下划线形式的名称，用于action
            this.options.pageNameActionForm = _.snakeCase(_.upperCase(this.options.pageName));
            this.options.date = util.getDate();
        });
    }

    _updateJsMap(repoType, pageName, jsMapPath) {
        try {
            let jsMapFileContent = this.fs.read(jsMapPath);
            let jsMapTree = program(jsMapFileContent);
            let pagesContent = jsMapTree.var('pages').value();

            let content2Add = repoType == util.REPOS.PC ?
                `'${pageName}'`
                : `'${pageName}/${pageName}.es6.js'`;
            pagesContent.push(content2Add);

            this.fs.write(jsMapPath, jsMapTree.toString());

        } catch (error) {
            this.log(chalk.red(`[✘]更新 jsMap 失败，请手动将新增的 ${chalk.red(pageName)} 添加到 fis-conf/jsMap.js 中`));
        }
    }

    writing() {

        let self = this;
        let tplOption = Object.assign({}, this.options);

        // 使用 copyTpl 复制整个文件夹，使用 registerTransformStream 来为某些文件单独命名
        this.registerTransformStream(rename((path) => {
            path.dirname = path.dirname.replace(/pageName/g, (match) => {
                return util.replacer(match, this.options);
            });
            path.basename = path.basename.replace(/pageName/g, (match) => {
                return util.replacer(match, this.options);
            });
            if (path.extname == '.dwt') {
                path.basename = this.options.repoType == util.REPOS.M ? _.snakeCase(path.basename) : _.camelCase(path.basename);
                path.extname = this.options.repoType == util.REPOS.M ? '.php' + path.extname : path.extname;
            }
        }));


        this.fs.copyTpl(
            this.templatePath(`${this.options.repoType}/pageName`),
            this.destinationPath('src/page/pageName'),
            tplOption
        );

        this.composeWith(require.resolve('../rp/index.js'),
            Object.assign({}, this.options, {
                routePageName: 'indexPage'
            })
        );

        this.composeWith(require.resolve('../cp/index.js'),
            Object.assign({}, this.options, {
                componentName: 'sampleComponent',
                componentType: 'SHARE'
            })
        );

        this.composeWith(require.resolve('../cp/index.js'),
            Object.assign({}, this.options, {
                componentName: 'sampleRoutePageComponent',
                componentType: 'PRIVATE',
                routePageName: 'indexPage'
            })
        );

        const jsMapPath = this.destinationPath('fis-conf/jsMap.js');
        this._updateJsMap(this.options.repoType, this.options.pageName, jsMapPath);
    }

    end() {
        this.log(chalk.green(`[✓]一切正常，主页面 ${chalk.red(this.options.pageName)} 相关文件生成完成!`));
    }
};
