'use strict';

const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const _ = require('lodash');
const util = require('../../misc/util.js');

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts);

        this.argument('componentName', { type: String, required: false, desc: "组件的名称" });
    }

    initializing() {
        //通过 composeWith 调用时会传递
        if (this.options.repoType) {
            return;
        }
        this.options.repoType = util.checkCwd(this);
    }

    _showComponentNamePrompt(answers) {
        return !this.options.componentName
            || util.isComponentExists(
                this.fs,
                this.options.repoType,
                answers.pageName || this.options.pagName,
                this.options.componentName);
    }

    prompting() {

        const prompts = [{
            type: 'input',
            name: 'author',
            store: true,
            message: '作者（用于生成文件头部的文档）:',
            default: util.getUserName(this),
            when: (answers) => {
                return !this.options.author;
            },
        }, {
            type: 'input',
            name: 'pageName',
            store: true,
            message: '组件将添加到哪个页面（dwt 所在的目录名）？',
            default: this.config.get('pageName'),
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                return !this.options.pageName;
            },
            validate: (input, answers) => {
                let pageName = input || this.options.pageName;
                if (!_.trim(input)) {
                    return '指定的页面名不能为空！';
                } else if (!util.isPageExists(this.fs, this.options.repoType, pageName)) {
                    //看是否能找到相应页面
                    return `指定的页面 ${input} 不存在，请检查输入或先使用 \`yo umu\` 生成`;
                } else {
                    return true;
                }
            }
        }, {
            type: 'list',
            name: 'componentType',
            choices: util.COMPONENT_TYPES,
            message: '多个路由页面共用还是单个路由页面使用的组件？',
            default: 1,
            when: (answers) => {
                return !this.options.componentType;
            },
        }, {
            type: 'list',
            name: 'routePageName',
            choices: (answers) => {
                //TODO: 测试为空的情况
                // return [];
                return util.expandDir(this.destinationPath(
                    `src/page/${this.options.pageName || answers.pageName}/routePage`
                ));
            },
            message: '请选择一个路由页',
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                return !this.options.routePageName
                    && answers.componentType == util.COMPONENT_TYPE_MAP.PRIVATE;
            }
        }, {
            type: 'input',
            name: 'componentName',
            message: '组件名称',
            default: this.options.componentName || '',
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                return this._showComponentNamePrompt(answers);
            },
            validate: (input, answers) => {
                let pageName = answers.pageName || this.options.pageName;

                if (!_.trim(input)) {
                    return chalk.red('路由页的名称不能为空！');
                } else if (util.isComponentExists(this.fs, this.options.repoType, pageName, input)) {
                    return chalk.red(`组件 ${input} 已经存在`);
                } else {
                    return true;
                }
            }
        }];

        return this.prompt(prompts).then(props => {
            this.options = Object.assign({}, this.options, props);

            this.options.pageName = _.camelCase(this.options.pageName);
            this.options.componentName = _.camelCase(this.options.componentName);
            //中划线形式的名称,用于样式类名
            this.options.componentNameClsForm = _.kebabCase(this.options.componentName);
            //用作class 类名
            this.options.componentNameComponentForm = _.startCase(this.options.componentName).replace(/\s+/g, '');

            this.options.date = this.options.date || util.getDate();
        });
    }

    writing() {

        let self = this;

        let tplOption = Object.assign({}, this.options);

        let srcPathPrefix = `${this.options.repoType}/componentName/`;
        let destinationPath = `src/page/${this.options.pageName}/components/${this.options.componentName}`;

        if (this.options.componentType == util.COMPONENT_TYPE_MAP.PRIVATE) {
            destinationPath = `src/page/${this.options.pageName}/routePage/${this.options.routePageName}/components/${this.options.componentName}`;
        }

        let filePostfix = '';
        if (this.options.repoType == util.REPOS.M) {
            filePostfix = '.es6';
        }

        this.fs.copyTpl(
            this.templatePath(`${srcPathPrefix}componentName${filePostfix}.js`),
            this.destinationPath(`${destinationPath}/${this.options.componentName}${filePostfix}.js`),
            tplOption
        );

        if (this.options.repoType == util.REPOS.M) {
            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}dataFlow.es6.js`),
                this.destinationPath(`${destinationPath}/dataFlow.es6.js`),
                tplOption
            );

            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}componentNameSaga.es6.js`),
                this.destinationPath(`${destinationPath}/${this.options.componentName}Saga.es6.js`),
                tplOption
            );
        }

        if (this.options.repoType == util.REPOS.PC) {
            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}componentName.html`),
                this.destinationPath(`${destinationPath}/${this.options.componentName}.html`),
                tplOption
            );
        }

        this.fs.copyTpl(
            this.templatePath(`${srcPathPrefix}componentName.css`),
            this.destinationPath(`${destinationPath}/${this.options.componentName}.css`),
            tplOption
        );

    }

    end() {
        this.log(chalk.green(`[✓]组件 ${chalk.red(this.options.componentName)} 生成完毕！`));
    }
};
