'use strict';

const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const _ = require('lodash');
const util = require('../../misc/util.js');

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts);

        this.argument('routePageName', { type: String, required: false, desc: '路由页面的名称' });

    }

    initializing() {
        //通过 composeWith 调用时会传递
        if (this.options.repoType) {
            return;
        }
        this.options.repoType = util.checkCwd(this);
    }

    _showRoutePageNamePrompt(answers) {
        return !this.options.routePageName
            || util.isRoutePageExists(
                this.fs,
                this.options.repoType,
                answers.pageName || this.options.pagName,
                this.options.routePageName);
    }

    prompting() {

        const prompts = [{
            type: 'input',
            name: 'author',
            store: true,
            message: '作者（用于生成文件头部的文档）:',
            default: util.getUserName(this),
            when: (answers) => {
                return !this.options.author;
            },
        }, {
            type: 'input',
            name: 'pageName',
            store: true,
            message: '路由页面将添加到哪个页面（dwt 所在的目录名）？',
            default: this.config.get('pageName'),
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                let pageName = this.options.pageName;
                return !this.options.pageName;
            },
            validate: (input, answers) => {
                let pageName = input || this.options.pageName;
                if (!_.trim(input)) {
                    return '指定的页面名不能为空！';
                } else if (!util.isPageExists(this.fs, this.options.repoType, pageName)) {
                    //看是否能找到相应页面
                    return `指定的页面${input}不存在，请检查输入或先使用 \`yo umu\` 生成`;
                } else {
                    return true;
                }
            }
        }, {
            type: 'input',
            name: 'routePageName',
            message: '页面名称，一些文件名也会基于此而生成',
            default: this.options.routePageName || '',
            filter: (input) => {
                return _.camelCase(input);
            },
            when: (answers) => {
                return this._showRoutePageNamePrompt(answers);
            },
            validate: (input, answers) => {
                let pageName = answers.pageName || this.options.pageName;

                if (!_.trim(input)) {
                    return chalk.red('路由页的名称不能为空！');
                } else if (util.isRoutePageExists(this.fs, this.options.repoType, pageName, input)) {
                    return chalk.red(`${input} 已经存在`);
                } else {
                    return true;
                }
            }
        }];

        return this.prompt(prompts).then(props => {
            this.options = Object.assign({}, this.options, props);

            this.options.pageName = _.camelCase(this.options.pageName);
            this.options.routePageName = _.camelCase(this.options.routePageName);
            //中划线形式的名称,用于样式类名
            this.options.routePageNameClsForm = _.kebabCase(this.options.routePageName);
            //用作class 类名
            this.options.routePageNameComponentForm = _.startCase(this.options.routePageName).replace(/\s+/g, '');
            this.options.date = this.options.date || util.getDate();
        });
    }

    writing() {

        let self = this;

        let tplOption = Object.assign({}, this.options);

        // 不能像主生成器中那样使用 copyTpl 加上 registerTransformStream 来批量复制同时定制文件名，
        // 因为此种方式在生成器以 composeWith 方式调用时不生效 see https://github.com/yeoman/generator/issues/819
        // 所以只能单个文件地复制
        let srcPathPrefix = `${this.options.repoType}/routePageName/`;
        let routePageDestPath = `src/page/${this.options.pageName}/routePage/${this.options.routePageName}/`;
        let reducerDestPath = `src/page/${this.options.pageName}/reducers/${this.options.routePageName}Reducer.es6.js`;

        let filePostfix = '';
        if (this.options.repoType == util.REPOS.M) {
            filePostfix = '.es6';
        }

        this.fs.copyTpl(
            this.templatePath(`${srcPathPrefix}routePageName${filePostfix}.js`),
            this.destinationPath(`${routePageDestPath}${this.options.routePageName}${filePostfix}.js`),
            tplOption
        );

        if (this.options.repoType == util.REPOS.M) {
            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}routePageNameReducer.es6.js`),
                this.destinationPath(reducerDestPath),
                tplOption
            );
        }

        if (this.options.repoType == util.REPOS.PC) {
            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}route.js`),
                this.destinationPath(`${routePageDestPath}route.js`),
                tplOption
            );
            this.fs.copyTpl(
                this.templatePath(`${srcPathPrefix}routePageName.html`),
                this.destinationPath(`${routePageDestPath}${this.options.routePageName}.html`),
                tplOption
            );
        }

        this.fs.copyTpl(
            this.templatePath(`${srcPathPrefix}routePageName.css`),
            this.destinationPath(`${routePageDestPath}${this.options.routePageName}.css`),
            tplOption
        );

    }

    end() {
        this.log(chalk.green(`[✓]路由页面 ${chalk.red(this.options.routePageName)} 生成完毕！`));
    }
};
